// created on 6/26/2004 at 1:42 PM
/*
 *   Copyright (c) 2004, Alexandros Frantzis (alf82 [at] freemail [dot] gr)
 *
 *   This file is part of Bless.
 *
 *   Bless is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   Bless is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Bless; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
using System.Collections;
using System;
using System.Xml;
using Gtk;
using NUnit.Framework;
using Bless.Gui.Areas;
using Bless.Gui.Drawers;
using Bless.Buffers;

namespace Bless.Gui {

///<summary>
/// A class that keeps information about how the data will be
/// placed and shown on screen.
///</summary>
public class Layout {

	ArrayList areas;
	XmlDocument layoutDoc;
	Drawer font;
	
	public ArrayList Areas {
		get {return areas;}
	}
	
	
	public Layout() 
	{
		areas=new ArrayList();
		layoutDoc=new XmlDocument();
	}
	
	public Layout(string name): this() 
	{
		Load(name);
	}

/*	public void InsertArea(int i, string name)
	{
		Area a=Area.Factory(name, drawingArea, byteBuffer, font);
		if (a!=null) {
			areas.Insert(i, a);
		} 
	}
	
	public void RemoveArea(int i)
	{
		if (i < areas.Count) {
			areas.RemoveAt(i);
		} 
	}
	
	public string GetAreaType(int i) 
	{
		if (i < areas.Count) {
			return ((Area)areas[i]).Type;
		}
		else
			return null; 
	}*/

	///<summary>Loads the layout from a file containing xml</summary>
	public void Load(string name) 
	{
		layoutDoc.Load(name);
	}
	
	///<summary>Loads the layout from string containing xml</summary>
	public void LoadXml(string xml)
	{
		layoutDoc.LoadXml(xml);
	}

	///<summary>Realizes the areas</summary>
	public void Realize(Gtk.DrawingArea da, ByteBuffer bb)
	{	
		XmlNodeList areaList = layoutDoc.GetElementsByTagName("area");
		
		foreach(XmlNode areaNode in areaList) {
			XmlAttributeCollection attrColl =  areaNode.Attributes;
			string type=attrColl["type"].Value;
			Area area=Area.Factory(type, da);
			if (area==null)
				continue;
			area.Buffer=bb;
			areas.Add(area);
			switch(type) {
				case "hexadecimal":
					RealizeHex(areaNode, (area as HexArea));
					break;
				case "decimal":
					RealizeDecimal(areaNode, (area as DecimalArea));
					break;
				case "octal":
					RealizeOctal(areaNode, (area as OctalArea));
					break;
				case "binary":
					RealizeBinary(areaNode, (area as BinaryArea));
					break;
				case "ascii":
					RealizeAscii(areaNode, (area as AsciiArea));
					break;
				case "separator":
					RealizeSeparator(areaNode, (area as SeparatorArea));
					break;
				case "offset":
					RealizeOffset(areaNode, (area as OffsetArea));
					break;					
				default:
					break;	
			}
		
		} // end foreach
		
		// give the focus to the first applicable area
		foreach(Area a in areas) {
			if (a.Type!="offset" && a.Type !="separator") {
				a.HasCursorFocus=true;
				break;
			}
		}
		
		// reset cursor
		foreach(Area a in areas) {
			a.CursorOffset=0;
			a.CursorDigit=0;		
		}
		
	}
		
	
	///<summary>Configures a hex area with the properties in the node</summary>
	void RealizeHex(XmlNode parentNode, HexArea area) 
	{
		Drawer.Information info=new Drawer.Information();
		
		XmlNodeList childNodes=parentNode.ChildNodes;
		foreach(XmlNode node in childNodes) {
			if (node.Name=="grouping")
				area.Grouping=Convert.ToInt32(node.InnerText);
			if (node.Name=="case")
				info.Uppercase=(node.InnerText=="upper");	
			if (node.Name=="display")
				ParseDisplay(node, info);
		}							
		
		area.Font=new HexDrawer(area.DrawingArea, info);
	}

	///<summary>Configure a decimal area with the properties in the node</summary>	
	void RealizeDecimal(XmlNode parentNode, DecimalArea area)
	{
		Drawer.Information info=new Drawer.Information();
		
		XmlNodeList childNodes=parentNode.ChildNodes;
		foreach(XmlNode node in childNodes) {
			if (node.Name=="grouping")
				area.Grouping=Convert.ToInt32(node.InnerText);
			if (node.Name=="display")
				ParseDisplay(node, info);
		}
		area.Font=new DecimalDrawer(area.DrawingArea, info);	
	
	}

	///<summary>Configure a octal area with the properties in the node</summary>		
	void RealizeOctal(XmlNode parentNode, OctalArea area)
	{
		Drawer.Information info=new Drawer.Information();
		
		XmlNodeList childNodes=parentNode.ChildNodes;
		foreach(XmlNode node in childNodes) {
			if (node.Name=="grouping")
				area.Grouping=Convert.ToInt32(node.InnerText);
			if (node.Name=="display")
				ParseDisplay(node, info);
		}
		area.Font=new OctalDrawer(area.DrawingArea, info);	
	
	}
	
	///<summary>Configure a binary area with the properties in the node</summary>
	void RealizeBinary(XmlNode parentNode, BinaryArea area)
	{
		Drawer.Information info=new Drawer.Information();
		
		XmlNodeList childNodes=parentNode.ChildNodes;
		foreach(XmlNode node in childNodes) {
			if (node.Name=="grouping")
				area.Grouping=Convert.ToInt32(node.InnerText);
			if (node.Name=="display")
				ParseDisplay(node, info);
		}
		area.Font=new BinaryDrawer(area.DrawingArea, info);	
	}
	
	///<summary>Configure an ascii area with the properties in the node</summary>
	void RealizeAscii(XmlNode parentNode, AsciiArea area)
	{
		Drawer.Information info=new Drawer.Information();

		XmlNodeList childNodes=parentNode.ChildNodes;
		foreach(XmlNode node in childNodes) {
			if (node.Name=="display")
				ParseDisplay(node, info);
		}
		area.Font=new AsciiDrawer(area.DrawingArea, info);
	}
	
	///<summary>Configure a separator area with the properties in the node</summary>
	void RealizeSeparator(XmlNode parentNode, SeparatorArea area)
	{
		Drawer.Information info=new Drawer.Information();
		
		XmlNodeList childNodes=parentNode.ChildNodes;
		foreach(XmlNode node in childNodes) {
			if (node.Name=="display")
				ParseDisplay(node, info);
		}
		area.Font=new DummyDrawer(area.DrawingArea, info);
	}
	
	///<summary>Configure an offset area with the properties in the node</summary>
	void RealizeOffset(XmlNode parentNode, OffsetArea area)
	{
		Drawer.Information info=new Drawer.Information();
		
		XmlNodeList childNodes=parentNode.ChildNodes;
		foreach(XmlNode node in childNodes) {
			if (node.Name=="case")
				info.Uppercase=(node.InnerText=="upper");
			if (node.Name=="display")
				ParseDisplay(node, info);
		}
		area.Font=new HexDrawer(area.DrawingArea, info);
	}
	
	///<summary>Parse the <fonts> tag in layout files</summary>
	void ParseDisplay(XmlNode parentNode, Drawer.Information info) 
	{
		XmlNodeList childNodes=parentNode.ChildNodes;
		foreach(XmlNode node in childNodes) {
			if (node.Name=="evenrow")
				ParseDisplayEven(node,info);
			else if (node.Name=="oddrow")
				ParseDisplayOdd(node,info);	
			else if (node.Name=="font")
				info.FontName=node.InnerText;
		}		
	}
	
	void ParseDisplayEven(XmlNode parentNode, Drawer.Information info)
	{
		string fg, bg;
		XmlNodeList childNodes=parentNode.ChildNodes;
		foreach(XmlNode node in childNodes) {
			if (node.Name=="evencolumn") {
				ParseDisplayType(node, out fg, out bg);
				if (bg!=null)
					info.rEvencEvenBg=bg;
				if (fg!=null)
					info.rEvencEvenFg=fg;
			}
			else if (node.Name=="oddcolumn") {
				ParseDisplayType(node, out fg, out bg);
				if (bg!=null)
					info.rEvencOddBg=bg;
				if (fg!=null)
					info.rEvencOddFg=fg;
			}
			else if (node.Name=="selectedcolumn") {
				ParseDisplayType(node, out fg, out bg);
				if (bg!=null)
					info.rEvencSelBg=bg;
				if (fg!=null)
					info.rEvencSelFg=fg;
			}
		}
	
	}
	
	void ParseDisplayOdd(XmlNode parentNode, Drawer.Information info)
	{
		string fg, bg;
		XmlNodeList childNodes=parentNode.ChildNodes;
		foreach(XmlNode node in childNodes) {
			if (node.Name=="evencolumn") {
				ParseDisplayType(node, out fg, out bg);
				if (bg!=null)
					info.rOddcEvenBg=bg;
				if (fg!=null)
					info.rOddcEvenFg=fg;
			}
			else if (node.Name=="oddcolumn") {
				ParseDisplayType(node, out fg, out bg);
				if (bg!=null)
					info.rOddcOddBg=bg;
				if (fg!=null)
					info.rOddcOddFg=fg;
			}
			else if (node.Name=="selectedcolumn") {
				ParseDisplayType(node, out fg, out bg);
				if (bg!=null)
					info.rOddcSelBg=bg;
				if (fg!=null)
					info.rOddcSelFg=fg;
			}
		}
	
	}
	
	///<summary>Parse a font type</summary>
	void ParseDisplayType(XmlNode parentNode, out string fg, out string bg)
	{
		fg=null;
		bg=null;
		XmlNodeList childNodes=parentNode.ChildNodes;
		foreach(XmlNode node in childNodes) {
			if (node.Name=="foreground")
				fg=node.InnerText;
			if (node.Name=="background")
				bg=node.InnerText;
		}
	}

}

}//namespace
