// created on 6/18/2004 at 7:46 PM
/*
 *   Copyright (c) 2004, Alexandros Frantzis (alf82 [at] freemail [dot] gr)
 *
 *   This file is part of Bless.
 *
 *   Bless is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   Bless is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Bless; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
using System;
using Gtk;
using Gdk;
using Bless.Gui.Drawers;

namespace Bless.Gui.Areas {

///<summary>An area that contains a vertical separator line</summary>
class SeparatorArea : Area {

	Gdk.GC rEvencEvenFg;
	
	public SeparatorArea(Gtk.DrawingArea da)
	: base(da) 
	{
		type="separator";
		rEvencEvenFg=new Gdk.GC(da.GdkWindow);
	}
	
	public override Drawer Font {
		set { 
			base.Font=value;
			Gdk.Color col=new Gdk.Color();
			Gdk.Color.Parse(font.Info.rEvencEvenFg, ref col); 
			rEvencEvenFg.RgbFgColor=col;
		}
	}
	
	protected override void RenderRow(int i, int p, int n, bool blank, bool select) 
	{
	/*
		if (n==0)
			return;
		Gdk.Window window = drawingArea.GdkWindow;
        Gdk.GC gc=new Gdk.GC(window);
		int ry=i*font.Height;
		int rx=x+font.Width/2;
		
        Gdk.Color green=new Gdk.Color(0,0x7f,0);
        gc.RgbFgColor=green;
        
        window.DrawLine(gc, rx, ry, rx, ry+font.Height);
	*/
	}
	
	public override void Scroll(long offset)
	{
		int nrows=height/font.Height;
		long bleft=nrows*bpr;
		int rfull=0;
		int blast=0;
		
		if (bpr>0) {
			if (bleft+offset > byteBuffer.Size)
				bleft=byteBuffer.Size - offset + 1;
		
			// calculate number of full rows
			// and number of bytes in last (non-full)
			rfull=(int)(bleft/bpr);
			blast=(int)(bleft%bpr);
		
			if (blast!=0)
				rfull++;
		}
		
		if (rfull==0)
			return;
		
		this.offset=offset;
			
		// draw seperator 
		backPixmap.DrawLine(rEvencEvenFg, x+font.Width/2, 0, x+font.Width/2, font.Height*rfull);
	
	}

	public override int CalcWidth(int n) 
	{
		return font.Width; 
	}
	
	public override void GetDisplayInfoByOffset(long off, out int orow, out int obyte, out int ox, out int oy)
	{	 
		orow=(int)((off-offset)/bpr);
		obyte=(int)((off-offset)%bpr);
			
		oy=orow*font.Height;
		
		ox=0;
	} 

	public override long GetOffsetByDisplayInfo(int x, int y, out int digit, out  GetOffsetFlags flags)
	{
		flags=0;
		int row=y/font.Height;
		long off=row*bpr+offset;
		if (off >= byteBuffer.Size)
			flags |= GetOffsetFlags.Eof;
			
		digit=0;
	
		return off;
	}
	
	public override void SetSelection(long start, long end)
	{
		SetSelectionNoRender(start, end);
	}
	
	public override void MoveCursor(long offset, int digit)
	{
		MoveCursorNoRender(offset, digit);
	}
}


}//namespace