// created on 1/10/2005 at 7:02 PM
/*
 *   Copyright (c) 2005, Alexandros Frantzis (alf82 [at] freemail [dot] gr)
 *
 *   This file is part of Bless.
 *
 *   Bless is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   Bless is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Bless; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
using System;
using Gtk;
using Bless.Tools.Find;
using Bless.Util;
using Bless.Gui;
using Bless.Gui.Areas;
using Bless.Buffers;
using System.Text;
 
namespace Bless.Gui.Dialogs {
  
class FindDialog :  Gtk.Window
{
	IFinder finder;
	bool patternChanged;
	
	[Glade.Widget]
	Gtk.Entry PatternEntry;
	[Glade.Widget]
	Gtk.VBox FindVBox;
	[Glade.Widget]
	Gtk.Button FindNextButton;
	[Glade.Widget]
	Gtk.Button FindPreviousButton;
	[Glade.Widget]
	Gtk.Entry InterpretEntry;
	
	public FindDialog(IFinder finder): base("Find")
	{
		Glade.XML gxml = new Glade.XML (null, "bless.glade", "FindVBox", null);
		gxml.Autoconnect (this);
		
		// setup window
		this.SkipTaskbarHint=true;
		this.TypeHint=Gdk.WindowTypeHint.Dialog;
		
		// create finder
		this.finder=finder;
		
		FindNextButton.Sensitive=false;
		FindPreviousButton.Sensitive=false;
		
		this.Add(FindVBox);
		this.Hide();
	}
	
	///<summary>Load the dialog with data from the DataView's selection</summary>
	public void LoadWithSelection(DataView dv)
	{
		ByteBuffer bb=dv.Buffer;
		
		// load selection only if it isn't very large 
		if (dv.Selection.Size <= 1024 && dv.Selection.Size > 0) {
			byte[] ba=new byte[dv.Selection.Size];
			for (int i=0;i<dv.Selection.Size;i++) {
				ba[i]=bb[dv.Selection.Start+i];
			}
		
			bool decodingOk=false;
			Area focusedArea=dv.FocusedArea;
			
			if (focusedArea!=null && focusedArea.Type=="ascii") {
				string result=Encoding.ASCII.GetString(ba);
				
				// if the byte sequence cannot be displayed correctly 
				// as ascii, eg it contains a zero byte, use hexadecimal
				if (result.IndexOf('\x00')!=-1)
					decodingOk=false;
				else {	
					InterpretEntry.Text="Text";
					PatternEntry.Text=result;
					decodingOk=true;
				}				
			}
			
			if (!decodingOk) {
				InterpretEntry.Text="Hexadecimal";
				PatternEntry.Text=ByteArray.ToHexString(ba);
			}
			
			PatternEntry.SelectRegion(0, PatternEntry.Text.Length);
		}
	}
	
	///<summary>Update the search pattern from the text entry</summary>
	void UpdatePattern()
	{
		if (!patternChanged)
			return;
		
		try {
			if (InterpretEntry.Text=="Hexadecimal")
				finder.Strategy.Pattern=ByteArray.FromHexString(PatternEntry.Text);
			else if (InterpretEntry.Text=="Text")
				finder.Strategy.Pattern=Encoding.ASCII.GetBytes(PatternEntry.Text); 
		}	
		catch (FormatException e) {
			ErrorAlert ea=new ErrorAlert("Invalid Search Pattern", e.Message, this);
			ea.Run();
			ea.Destroy();
			throw;
		}
		patternChanged=false;
	}
	
	void OnPatternEntryChanged(object o, EventArgs args)
	{
		string pat=PatternEntry.Text;
		if (pat.Length==0) {
			FindNextButton.Sensitive=false;
			FindPreviousButton.Sensitive=false;
		}
		else {
			FindNextButton.Sensitive=true;
			FindPreviousButton.Sensitive=true;
		}
		patternChanged=true;
	}
	
	void OnInterpretEntryChanged(object o, EventArgs args)
	{
		patternChanged=true;
	}
	
	void OnFindNextClicked(object o, EventArgs args)
	{
		try {
			UpdatePattern();
			if (!finder.FindNext()) {
				InformationAlert ia=new InformationAlert("The pattern you requested was not found.", "", this);
				ia.Run();
				ia.Destroy();
			}
		}
		catch(FormatException) { }
	}
	
	void OnFindPreviousClicked(object o, EventArgs args)
	{
		try {
			UpdatePattern();
			if (!finder.FindPrevious()) {
				InformationAlert ia=new InformationAlert("The pattern you requested was not found.", "", this);
				ia.Run();
				ia.Destroy();
			}
		}
		catch(FormatException) { }
	}
	
	void OnCloseClicked(object o, EventArgs args)
	{
		this.Hide();
	}
	
	protected override bool OnDeleteEvent(Gdk.Event e)
	{
		this.Hide();
		return true;
	}
}
 
} //end namespace