// created on 6/28/2004 at 12:58 PM
/*
 *   Copyright (c) 2004, Alexandros Frantzis (alf82 [at] freemail [dot] gr)
 *
 *   This file is part of Bless.
 *
 *   Bless is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   Bless is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Bless; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
using System;
using System.IO;
using Gtk;
using Glade;
using Bless.Gui.Areas;
using Bless.Buffers;
 
namespace Bless.Gui.Dialogs { 

///<summary>
/// A dialog that lets the user select one of the available layouts
///</summary>
public class LayoutSelectionDialog {
	
	DataView dataView;
	DataView dataPreview;
	[Glade.Widget] Gtk.Button OKButton;
	[Glade.Widget] Gtk.Button CancelButton;
	[Glade.Widget] Gtk.TreeView LayoutList;
	[Glade.Widget] Gtk.Frame PreviewFrame;
	[Glade.Widget] Gtk.Window LayoutSelectionWindow;
	TreeIter selectedLayoutIter;
	string selectedLayout;
	string layoutDir;
	
	public LayoutSelectionDialog(DataView dv) 
	{
		Glade.XML gxml = new Glade.XML (null, "bless.glade", "LayoutSelectionWindow", null);
		gxml.Autoconnect (this);
		
		dataView=dv;
		
		// create the preview area
		dataPreview=new DataView();
		ByteBuffer bb=new ByteBuffer();
		bb.Append(new byte[]{0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15});
		bb.Append(new byte[]{16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31});
		dataPreview.Buffer=bb;
		
		PreviewFrame.Add(dataPreview.Display);
		PreviewFrame.ShowAll();
		
		layoutDir=System.Environment.GetEnvironmentVariable("HOME")+"/.bless/layouts/";
		
		// Initialize list
		PopulateLayoutList();
	}
	
	///<summary>Populate the layout list</summary>
	void PopulateLayoutList()
	{
		ListStore ls=new ListStore(typeof(string));
		
		
		// fill list
		if (Directory.Exists(layoutDir)) {
			string[] files=Directory.GetFiles(layoutDir,"*.layout");	
			foreach (string s in files) {
				ls.AppendValues(System.IO.Path.GetFileName(s));
			}
		}
		
		LayoutList.Model=ls;
		LayoutList.AppendColumn("Layout",new CellRendererText (), "text", 0); 
		LayoutList.Selection.Changed +=OnLayoutListSelectionChanged;
	}
	
	///<summary>Handle list selection changed event</summary>
	public void OnLayoutListSelectionChanged (object o, EventArgs args) 
	{
		TreeSelection sel=(TreeSelection)o;
		TreeModel tm;
		TreeIter ti;
		
		if (sel.GetSelected(out tm, out ti)) {
			string val = (string) tm.GetValue (ti, 0);
			try {
				dataPreview.Display.Layout=new Layout(layoutDir+val);
				selectedLayoutIter=ti;
				selectedLayout=val;
			}
			catch(System.Xml.XmlException ex) {
				ErrorAlert ea=new ErrorAlert("Error parsing layout file " + val, ex.Message, LayoutSelectionWindow);
				ea.Run();
				ea.Destroy();
				if (selectedLayout != null)
					sel.SelectIter(selectedLayoutIter);
				else
					sel.UnselectIter(ti);
				dataPreview.Display.Redraw();
			}
			
		}
	}
	
	
	public void OnOKButtonClicked(object o, EventArgs args)
	{
		if (selectedLayout!=null)
			dataView.Display.Layout=new Layout(layoutDir + selectedLayout);
		// dispose preview Area pixmaps
		dataPreview.Display.DisposePixmaps();
				
		LayoutSelectionWindow.Destroy();
	}
	
	public void OnCancelButtonClicked(object o, EventArgs args)
	{
		// dispose preview Area pixmaps
		dataPreview.Display.DisposePixmaps();
		LayoutSelectionWindow.Destroy();
	}
}


}//namespace