// created on 2/12/2005 at 12:33 PM
/*
 *   Copyright (c) 2005, Alexandros Frantzis (alf82 [at] freemail [dot] gr)
 *
 *   This file is part of Bless.
 *
 *   Bless is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   Bless is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Bless; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
using System;
using Gtk;
using Bless.Tools.Find;
using Bless.Util;
using Bless.Gui;
using Bless.Gui.Areas;
using Bless.Buffers;
using System.Text;
 
namespace Bless.Gui.Dialogs {
  
class ReplaceDialog :  Gtk.Window
{
	IFinder finder;
	
	bool searchPatternChanged;
	bool replacePatternChanged;
	
	byte[] replacePattern;
	
	
	[Glade.Widget] Gtk.VBox ReplaceVBox;
	[Glade.Widget] Gtk.Button FindButton;
	[Glade.Widget] Gtk.Button ReplaceButton;
	[Glade.Widget] Gtk.Button ReplaceAllButton;
	
	[Glade.Widget] Gtk.Entry SearchPatternEntry;
	[Glade.Widget] Gtk.Entry SearchInterpretEntry;
	[Glade.Widget] Gtk.Entry ReplacePatternEntry;
	[Glade.Widget] Gtk.Entry ReplaceInterpretEntry;
	
	
	public ReplaceDialog(IFinder finder): base("Replace")
	{
		Glade.XML gxml = new Glade.XML (null, "bless.glade", "ReplaceVBox", null);
		gxml.Autoconnect (this);
		
		// setup window
		this.SkipTaskbarHint=true;
		this.TypeHint=Gdk.WindowTypeHint.Dialog;
		
		this.finder=finder;
		
		FindButton.Sensitive=false;
		ReplaceButton.Sensitive=false;
		ReplaceAllButton.Sensitive=false;
		
		replacePattern=new byte[0];
		
		this.Add(ReplaceVBox);
		this.Hide();
	}
	
	///<summary>Load the dialog with data from the DataView's selection</summary>
	public void LoadWithSelection(DataView dv)
	{
		ByteBuffer bb=dv.Buffer;
		
		// load selection only if it isn't very large 
		if (dv.Selection.Size <= 1024 && dv.Selection.Size > 0) {
			byte[] ba=new byte[dv.Selection.Size];
			for (int i=0;i<dv.Selection.Size;i++) {
				ba[i]=bb[dv.Selection.Start+i];
			}
		
			bool decodingOk=false;
			Area focusedArea=dv.FocusedArea;
			
			if (focusedArea!=null && focusedArea.Type=="ascii") {
				string result=Encoding.ASCII.GetString(ba);
				
				// if the byte sequence cannot be displayed correctly 
				// as ascii, eg it contains a zero byte, use hexadecimal
				if (result.IndexOf('\x00')!=-1)
					decodingOk=false;
				else {	
					SearchInterpretEntry.Text="Text";
					SearchPatternEntry.Text=result;
					decodingOk=true;
				}				
			}
			
			if (!decodingOk) {
				SearchInterpretEntry.Text="Hexadecimal";
				SearchPatternEntry.Text=ByteArray.ToHexString(ba);
			}
			
			SearchPatternEntry.SelectRegion(0, SearchPatternEntry.Text.Length);
			
			finder.LastFound=new Bless.Util.Range(dv.Selection);
		}
	}
	
	//
	// Search related methods
	//
	
	///<summary>Update the search pattern from the text entry</summary>
	void UpdateSearchPattern()
	{
		if (!searchPatternChanged)
			return;
			
		try {
			if (SearchInterpretEntry.Text=="Hexadecimal")
				finder.Strategy.Pattern=ByteArray.FromHexString(SearchPatternEntry.Text);
			else if (SearchInterpretEntry.Text=="Text")
				finder.Strategy.Pattern=Encoding.ASCII.GetBytes(SearchPatternEntry.Text); 
		}	
		catch (FormatException e) {
			ErrorAlert ea=new ErrorAlert("Invalid Search Pattern", e.Message, this);
			ea.Run();
			ea.Destroy();
			throw;
		}
		searchPatternChanged=false;
	}
	
	void OnSearchPatternEntryChanged(object o, EventArgs args)
	{
		string pat=SearchPatternEntry.Text;
		if (pat.Length==0) {
			FindButton.Sensitive=false;
			ReplaceButton.Sensitive=false;
			ReplaceAllButton.Sensitive=false;
		}
		else {
			FindButton.Sensitive=true;
			ReplaceButton.Sensitive=true;
			ReplaceAllButton.Sensitive=true;
		}
		searchPatternChanged=true;
	}
	
	void OnSearchInterpretEntryChanged(object o, EventArgs args)
	{
		searchPatternChanged=true;
	}
	
	void OnFindClicked(object o, EventArgs args)
	{
		try {
			UpdateSearchPattern();
			if (!finder.FindNext()) {
				InformationAlert ia=new InformationAlert("The pattern you requested was not found.", "", this);
				ia.Run();
				ia.Destroy();
			}
		}
		catch(FormatException) { }	
	}
	
	//
	// Replace related methods
	//
	
	///<summary>Update the replace pattern from the text entry</summary>
	void UpdateReplacePattern()
	{
		if (!replacePatternChanged)
			return;
			
		try {
			if (ReplaceInterpretEntry.Text=="Hexadecimal")
				replacePattern=ByteArray.FromHexString(ReplacePatternEntry.Text);
			else if (ReplaceInterpretEntry.Text=="Text")
				replacePattern=Encoding.ASCII.GetBytes(ReplacePatternEntry.Text); 
		}	
		catch (FormatException e) {
			ErrorAlert ea=new ErrorAlert("Invalid Replace Pattern", e.Message, this);
			ea.Run();
			ea.Destroy();
			throw;
		}
		replacePatternChanged=false;
	}
	
	void OnReplacePatternEntryChanged(object o, EventArgs args)
	{
		replacePatternChanged=true;
	}
	
	void OnReplaceInterpretEntryChanged(object o, EventArgs args)
	{
		replacePatternChanged=true;
	}
	
	
	void OnReplaceClicked(object o, EventArgs args)
	{
		try {
			UpdateSearchPattern();
			UpdateReplacePattern();
			finder.Replace(replacePattern);
			finder.FindNext();
		}
		catch(FormatException) { }	
	}
	
	void OnReplaceAllClicked(object o, EventArgs args)
	{
		try {
			UpdateSearchPattern();
			UpdateReplacePattern();
			int replaced=finder.ReplaceAll(replacePattern);
			InformationAlert ia=new InformationAlert("Found and replaced "+replaced+" occurences.", "", this);
			ia.Run();
			ia.Destroy();
		}
		catch(FormatException) { }	
	}
	
	void OnCloseClicked(object o, EventArgs args)
	{
		this.Hide();
	}
	
	protected override bool OnDeleteEvent(Gdk.Event e)
	{
		this.Hide();
		return true;
	}
}
 
} //end namespace
