// created on 6/14/2004 at 10:55 PM
/*
 *   Copyright (c) 2004, Alexandros Frantzis (alf82 [at] freemail [dot] gr)
 * 
 *   This file is part of Bless.
 *
 *   Bless is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   Bless is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Bless; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using System;
using Gtk;
using Gdk;
using Pango;
using NUnit.Framework;

namespace Bless.Gui.Drawers {



///<summary>Fast font drawing class</summary>
public abstract class Drawer {

	public class Information {
		public string FontName;
		public string FontLanguage;
		
		public string rEvencEvenFg;
		public string rEvencEvenBg;
		public string rEvencOddFg;
		public string rEvencOddBg;
		public string rEvencSelFg;
		public string rEvencSelBg;
		
		public string rOddcEvenFg;
		public string rOddcEvenBg;
		public string rOddcOddFg;
		public string rOddcOddBg;
		public string rOddcSelFg;
		public string rOddcSelBg;
		
		
		public bool Uppercase;
		
		public Information() 
		{
			FontName="Courier 12";
			FontLanguage="utf-8";
			
			rEvencEvenFg="black";
			rEvencEvenBg="white";
			
			rEvencOddFg="blue";
			rEvencOddBg="white";
			
			
			rOddcEvenFg="black";
			rOddcEvenBg="white";
			
			rOddcOddFg="blue";
			rOddcOddBg="white";
			
			rEvencSelFg="white";
			rEvencSelBg="#7b96ac";
			
			rOddcSelFg="white";
			rOddcSelBg="#7b96ac";
			
			
			Uppercase=false;
		}
		
	}
	
	public enum RowType {Even, Odd};
	public enum ColumnType {Even, Odd, Selected};
	
	// the widget the font will finally 
	// be printed on (used for info only)
	protected Gtk.Widget widget;
	protected Pango.FontDescription fontDescription;
	protected Information info;
	
	protected Gdk.Pixmap rEvencEven;
	protected Gdk.Pixmap rEvencOdd;
	protected Gdk.Pixmap rEvencSel;
	
	protected Gdk.Pixmap rOddcEven;
	protected Gdk.Pixmap rOddcOdd;
	protected Gdk.Pixmap rOddcSel;
	
	protected int width;
	protected int height;
	
	///<summary>Constructor</summary>
	public Drawer(Gtk.Widget wid, Information inf)
	{
		widget=wid;
		info=inf;
		
		fontDescription=Pango.FontDescription.FromString(info.FontName);
		Pango.Language lang=Pango.Language.FromString(info.FontLanguage);
		
		Pango.Context pangoCtx=widget.PangoContext;
		Pango.FontMetrics fm=pangoCtx.GetMetrics(fontDescription, lang);
		
		// set the font height and width
		height =(int)( (fm.Ascent+fm.Descent)/Pango.Scale.PangoScale);
    	width =(int) ((fm.ApproximateCharWidth)/Pango.Scale.PangoScale);

		// create the font pixmaps
		Gdk.Color colorFg=new Gdk.Color();
		Gdk.Color colorBg=new Gdk.Color();
		
		//even rows
		Gdk.Color.Parse(info.rEvencEvenFg, ref colorFg);
		Gdk.Color.Parse(info.rEvencEvenBg, ref colorBg);
		rEvencEven=Create(colorFg, colorBg);
		
		Gdk.Color.Parse(info.rEvencOddFg, ref colorFg);
		Gdk.Color.Parse(info.rEvencOddBg, ref colorBg);
		rEvencOdd=Create(colorFg, colorBg);
		
		Gdk.Color.Parse(info.rEvencSelFg, ref colorFg);
		Gdk.Color.Parse(info.rEvencSelBg, ref colorBg);
		rEvencSel=Create(colorFg, colorBg);
		
		//odd rows
		Gdk.Color.Parse(info.rOddcEvenFg, ref colorFg);
		Gdk.Color.Parse(info.rOddcEvenBg, ref colorBg);
		rOddcEven=Create(colorFg, colorBg);
				
		Gdk.Color.Parse(info.rOddcOddFg, ref colorFg);
		Gdk.Color.Parse(info.rOddcOddBg, ref colorBg);
		rOddcOdd=Create(colorFg, colorBg);
		
		Gdk.Color.Parse(info.rOddcSelFg, ref colorFg);
		Gdk.Color.Parse(info.rOddcSelBg, ref colorBg);
		rOddcSel=Create(colorFg, colorBg);

	}
	
	///<summary>Creates a pixmap with the drawn data</summary>
	abstract protected Gdk.Pixmap Create(Gdk.Color fg, Gdk.Color bg);
	
	///<summary>Draws the a byte</summary>
	abstract public void Draw(Gdk.GC gc, Gdk.Drawable dest, int x, int y, byte b, RowType rtype, ColumnType ctype); 
 
	///<summary>Select the pixmap according to the type</summary>
	protected Gdk.Pixmap SelectPixmap(RowType rtype, ColumnType ctype)
	{
		Gdk.Pixmap pix=null;
 
		if (rtype == RowType.Even) {
			if (ctype==ColumnType.Even) 
				pix=rEvencEven;
			else if (ctype==ColumnType.Odd)
				pix=rEvencOdd;
			else /*if (ctype==ColumnType.Selected)*/
				pix=rEvencSel;
		}
		else if (rtype == RowType.Odd) {
			if (ctype==ColumnType.Even) 
				pix=rOddcEven;
			else if (ctype==ColumnType.Odd)
				pix=rOddcOdd;
			else /*if (ctype==ColumnType.Selected)*/
				pix=rOddcSel;
		}
		
		return pix;
	}
	
	public void DisposePixmaps()
	{
		if (rEvencEven != null)
			rEvencEven.Dispose();
		if (rEvencOdd != null)
			rEvencOdd.Dispose();
		if (rEvencSel != null)
			rEvencSel.Dispose();
	
		if (rOddcEven != null)
			rOddcEven.Dispose();
		if (rOddcOdd != null)
			rOddcOdd.Dispose();
		if (rOddcSel != null)
			rOddcSel.Dispose();
	}
	
	public int Width{
		get { return width; }
	}
	
	public int Height{
		get { return height; }
	}
	
	public Drawer.Information Info{
		get { return info; }
	}

}

//<summary>dummy</summary>
public class DummyDrawer : Drawer {
	
	public DummyDrawer(Gtk.Widget wid, Information inf)
	:base(wid, inf)
	{
	}
	
	public override void Draw(Gdk.GC gc, Gdk.Drawable dest, int x, int y, byte b, RowType rtype, ColumnType ctype) 
	{
		
	}
	
	protected override Gdk.Pixmap Create(Gdk.Color fg, Gdk.Color bg)
	{
		return null;
	}
	
	

}

/*
#region Tests

[TestFixture]
public class FontDataTests {

	Drawer font;
	
	public static void Main() {
		FontDataTests fdt=new FontDataTests();
		fdt.Test1();
	}
	
	[Test]
	public void Test1() {
		Application.Init ();
		Gtk.Window win = new Gtk.Window ("Layout sample");
		DrawingArea da=new DrawingArea();
		win.SetDefaultSize (400, 300);
		win.DeleteEvent += OnWinDelete;
		
		da.Realized += OnRealized;
		da.ExposeEvent += OnExposed;
 
		win.Add (da);
		win.ShowAll ();
		Application.Run ();		
	}

	void OnExposed (object o, ExposeEventArgs args)
	{
		DrawingArea da=(DrawingArea)o;
		   
		for (int i=0; i<256; i++) {
			byte b=(byte)i;
			font.DrawHex(da.Style.BlackGC, da.GdkWindow, i*3*font.Width, 0, b, 
						(Drawer.Type)(i%2));
			font.DrawAscii(da.Style.BlackGC, da.GdkWindow, i*3*font.Width+1, font.Height, b, 
						(Drawer.Type)(i%2));
		}
	}
	
	void OnRealized (object o, EventArgs args) {
		font=new Drawer((Widget)o, "Courier 12", "utf-8");
	}
 
	void OnWinDelete (object o, DeleteEventArgs args)
	{
		Application.Quit ();
	}

} 




#endregion
*/


}





