// created on 1/18/2005 at 12:52 PM
/*
 *   Copyright (c) 2005, Alexandros Frantzis (alf82 [at] freemail [dot] gr)
 *
 *   This file is part of Bless.
 *
 *   Bless is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   Bless is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Bless; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
using System;
using NUnit.Framework;
 
namespace Bless.Util {

public class LongParser
{
	private LongParser() { }
	
	public static long ParseOctal(string s)
	{
		int i=0;
		int len=s.Length;
		
		if (len==0)
			throw new FormatException("The string to parse is empty.");
			
		// ignore leading whitespace
		while (i<len && s[i]==' ') i++;
			
		return ParseOctalInternal(s, i);
	}
	
	public static long ParseHexadecimal(string s)
	{
		int i=0;
		int len=s.Length;
		
		if (len==0)
			throw new FormatException("The string to parse is empty.");
		
		// ignore leading whitespace
		while (i<len && s[i]==' ') i++;
		
		// ignore '0x' if present
		if (i+1<len && s[i]=='0' && s[i+1]=='x')
			i+=2;
			
		return ParseHexadecimalInternal(s, i);
	}
	
	public static long ParseDecimal(string s)
	{
		int i=0;
		int len=s.Length;
		
		if (len==0)
			throw new FormatException("The string to parse is empty.");
		
		// ignore leading whitespace
		while (i<len && s[i]==' ') i++;
		
		return ParseDecimalInternal(s, i);
	}
	
	public static long Parse(string s)
	{
		int i=0;
		int len=s.Length;
		
		if (len==0)
			throw new FormatException("The string to parse is empty.");
		
		// ignore leading whitespace
		while (i<len && s[i]==' ') i++;
		
		//decide if it is hex, octal or decimal
		
		// if it starts with '0x' suppose it is hex
		if (i+1<len && s[i]=='0' && s[i+1]=='x')
			return ParseHexadecimalInternal(s, i+2);
	
		if (i<len && s[i]=='0')
			return ParseOctalInternal(s, i);
			
		return ParseDecimalInternal(s, i);
	}
 
	private static long ParseOctalInternal(string s, int pos)
	{
		long res=0;
		int len=s.Length;
			
		while (pos<len) {
			char c=s[pos];
			if (Char.IsWhiteSpace(c))
				break;
			else if (c<'0' || c>'7')
				throw new FormatException(String.Format("The number seems to be octal but contains invalid digit '{0}'.",c));
			res = 8*res + c-'0';
			pos++;
		}
		
		return res;
	}
	
	public static long ParseHexadecimalInternal(string s, int pos)
	{
		long res=0;
		int len=s.Length;
		
		while (pos<len) {
		char c=s[pos];
			if (Char.IsDigit(c))
				res = 16*res + c-'0';
			else if (c>='a' && c<='f')
				res = 16*res + c-'a'+10;
			else if (c>='A' && c<='F')
				res = 16*res + c-'A'+10;
			else if (Char.IsWhiteSpace(c))
				break;
			else		
				throw new FormatException(String.Format("The number seems to be hexadecimal but contains invalid digit '{0}'.",c));
			
			pos++;
		}
		
		return res;
	}
	
	public static long ParseDecimalInternal(string s, int pos)
	{
		long res=0;
		int len=s.Length;
		
		while (pos<len) {
			char c=s[pos];
			if (Char.IsDigit(c))
				res = 10*res + c-'0';
			else if (Char.IsWhiteSpace(c))
				break;
			else		
				throw new FormatException(String.Format("The number seems to be decimal but contains invalid digit '{0}'.",c));
			
			pos++;
		}
		
		return res;
	}
 
}

[TestFixture]
public class LongParserTests
{
	[Test]
	public void LongParserTest1()
	{
		long l=LongParser.Parse("  0x66  ");
		
		Assert.AreEqual(0x66, l,"#0");	
	}
	
	[Test]
	public void LongParserTest2()
	{
		long l=LongParser.Parse(" 066 ");
		
		Assert.AreEqual(54, l,"#0");	
	}
	
	
	[Test]
	public void LongParserTest3()
	{
		long l=LongParser.Parse("  66 ");
		
		Assert.AreEqual(66, l,"#0");	
	}
	
	[Test]
	[ExpectedException(typeof(FormatException))]
	public void LongParserTest4()
	{
		long l=LongParser.Parse("  096 ");
	}
	
	[Test]
	[ExpectedException(typeof(FormatException))]
	public void LongParserTest5()
	{
		long l=LongParser.Parse("  0x4r ");
	}
	
}

} // end namespace
 